#
#  Bounded stack language for the BSL editor
#
BSL type finite-state trans-line

typefaces
r---i	BSL-Plain
rb-R-	BSL-Wildcard
r--G-	BSL-CharacterRange
r--b-	BSL-Set
rbu--	BSL-Header
r--B-	BSL-Style
r-u--	BSL-Action
i--l-	BSL-Comment

state set
I	BSL-Plain
W	BSL-Plain
D	BSL-Plain
E	BSL-Plain
R	BSL-Plain
M	BSL-Plain
O	BSL-CharacterRange
S	BSL-Plain
C	BSL-Plain

rules
#
#  Initial states.
#  
#  I: BSL header, typefaces, state set
#  D: demo string
#  W: word range
#
I	\{BSL-Header}^BSL type:^typefaces:^state set	nop
I	\{BSL-Header}^demo string	goto D
I	\{BSL-Style}[A-Z][a-zA-Z-_]+[a-zA-Z]	nop
ID	\{BSL-Header}^word range	goto W
W	[^-]\{BSL-CharacterRange}-\{BSL-Plain}[^-]	nop
IDW	\{BSL-Header}^rules	goto S
#
#  Rules section.
#  
#  S: state list
#
S	\t	goto E
#
#  E/R/D/O: expression body and ranges (see spattern.preset.bsl)
#
E	[^:\\\[\]?*&^$\t]+	nop
E	\{BSL-Wildcard}\\{\{BSL-Style}[A-Z][a-zA-Z-_]*\{BSL-Wildcard}}	nop
E	\{BSL-Set}\:	nop
E	\{BSL-Wildcard}\\[()<>nt]	nop
E	\{BSL-Wildcard}\\\{BSL-Plain}?	nop
E	\{BSL-Wildcard}\?:\!:\*:\^:\$	nop
E	\{BSL-Wildcard}\&[0-9]\{BSL-CharacterRange}[/]!	push R, push R
E	\{BSL-Wildcard}\&[0-9]	nop
E	\{BSL-CharacterRange}[\[]!\^	goto O, push R
E	\{BSL-CharacterRange}[\[]!	goto O, push R
RM	\{BSL-CharacterRange}&0/[/]	pop
RM	\{BSL-Wildcard}\\[nt]	goto M
RM	\{BSL-Wildcard}\\\{BSL-Plain}?	goto M
R	?	goto M
M	\{BSL-CharacterRange}-\{BSL-Wildcard}\\[nt]	goto R
M	\{BSL-CharacterRange}-\{BSL-Wildcard}\\\{BSL-Plain}?	goto R
M	\{BSL-CharacterRange}-\{BSL-Plain}?	goto R
O	[1*+?!]	goto E
O		goto E
#
#  Recovery from incomplete expressions (not completely satisfactory)
#
EO	\t	goto C
EO	$	goto S
RM	\t	pop, goto C
RM	$	pop
#
#  C: one or more comma-separated actions
#
C	\{BSL-Action}goto:push:pop:nop	nop
C	$	goto S
#
#  Comments
#
IWDSC	\{BSL-Comment}^#*$	nop
