#
#  Bounded stack language for LaTeX
#
#  V1.53 - fixes for escaped %, \, &, {, }.
#  V1.50
#
BSL type recursive trans-line

typefaces
r----	LaTeX-Roman
i----	LaTeX-Italic
rb---	LaTeX-Bold
t----	LaTeX-Typewriter
l----	LaTeX-Large
r--B-	LaTeX-MathRoman
rb-B-	LaTeX-MathBold
i--B-	LaTeX-MathItalic
t--G-	LaTeX-Comment
rb-R-	LaTeX-Command
r--y-	LaTeX-Trick

state set
R	LaTeX-Roman
I	LaTeX-Italic
B	LaTeX-Bold
T	LaTeX-Typewriter
L	LaTeX-Large
r	LaTeX-MathRoman
i	LaTeX-MathRoman
b	LaTeX-MathBold
A	LaTeX-Roman
V	LaTeX-Typewriter
W	LaTeX-Typewriter

demo string The {\em expression} $2x + f(y)$ is not {\tt constant}

word range a-zA-Z@*

rules
#
#  Scan  sequences  of  non-  or hardly  special  characters  first (for
#  speed); in math mode, render only letters as italics.
#
RIBTL	[^\\{}%$@&]+		nop
RIBTLirb	\{LaTeX-Command}\&:@		nop
     i	\{LaTeX-MathItalic}[a-zA-Z]+		nop
     irb	[^a-zA-Z\\{}%$@&]+		nop
     irb	\{LaTeX-Command}[_^]		nop
#
#  Grouping
#
R       A	\{LaTeX-Command}{		push R
 IBTLirb	\{LaTeX-Command}{		push
RIBTLirb	\{LaTeX-Command}}		pop
RIBTL	\{LaTeX-Command}\$		push i
     irb	\{LaTeX-Command}\$		pop
#
#  Commands
#
RIBTLirb	\{LaTeX-Command}\\part[*]?{:\\chapter[*]?{:\\section[*]?{:\\subsection[*]?{	push L
RIBTLirb	\{LaTeX-Command}\\subsubsection[*]?{				push B
RIBTLirb	\{LaTeX-Command}\\begin{\{LaTeX-Roman}verbatim\{LaTeX-Command}}		push V
RIBTL	\{LaTeX-Command}\\begin{\{LaTeX-Roman}equation\{LaTeX-Command}}		push i, push A
RIBTL	\{LaTeX-Command}\\begin{\{LaTeX-Roman}eqnarray[*]?\{LaTeX-Command}}		push i, push A
RIBTLirb	\{LaTeX-Command}\\begin{\{LaTeX-Roman}[a-zA-Z@*]*\{LaTeX-Command}}		push, push A
RIBTL	\{LaTeX-Command}\\rm\>				goto R
     i b	\{LaTeX-Command}\\rm\>				goto r
RIBTL	\{LaTeX-Command}\\it\>				goto I
      rb	\{LaTeX-Command}\\it\>				goto i
RIBTL	\{LaTeX-Command}\\bf\>				goto B
     ir	\{LaTeX-Command}\\bf\>				goto b
RIBTLirb	\{LaTeX-Command}\\tt\>				goto T
RIBT irb	\{LaTeX-Command}\\large:\\Large:\\huge:\\Huge				goto L
R BTL	\{LaTeX-Command}\\em\>				goto I
 I	\{LaTeX-Command}\\em\>				goto R
      rb	\{LaTeX-Command}\\em\>				goto i
     i	\{LaTeX-Command}\\em\>				goto r
RIBTL	\{LaTeX-Command}\\\[:\\(				push i
     irb	\{LaTeX-Command}\\\]:\\)				pop
     irb	\{LaTeX-Command}\\end{\{LaTeX-Roman}equation\{LaTeX-Command}}		pop
     irb	\{LaTeX-Command}\\end{\{LaTeX-Roman}eqnarray[*]?\{LaTeX-Command}}		pop
RIBTLirb	\{LaTeX-Command}\\end{\{LaTeX-Roman}[a-zA-Z@*]*\{LaTeX-Command}}		pop
         V	\{LaTeX-Command}\\end{\{LaTeX-Roman}verbatim\{LaTeX-Command}}		pop
RIBTLirb	\{LaTeX-Command}\\verb!				push W
RIBTLirb	\{LaTeX-Command}\\[a-zA-Z@*]+				push A
RIBTLirbA	\{LaTeX-Command}\\%:\\\\:\\\$:\\\&:\\{:\\}				nop
RIBTLirb	\{LaTeX-Command}\\				nop
          W	\{LaTeX-Command}&0				pop
#
#  Comments
#
#  Include  a fix against the theoretical impossibility (unless we would
#  use   LaTeX  itself  to  highlight  the  source)  to  decide  whether
#  user-defined  commands such as theorems introduce a group in roman or
#  emphasized.
#
R BTLirbA	\{LaTeX-Command}%\{LaTeX-Trick}em\>\{LaTeX-Comment}*$	goto I
 I	\{LaTeX-Command}%\{LaTeX-Trick}em\>\{LaTeX-Comment}*$	goto R
RIBTLirbA	\{LaTeX-Command}%\{LaTeX-Comment}*$		nop
#
#  Command arguments (note the rule under Grouping)
#
        A	\{LaTeX-Command}\[\{LaTeX-Roman}[^\]]+\{LaTeX-Command}\]	nop
        A			pop

